@@ -18,25 +18,23 @@
 #include "mz_zip.h"
 #include "mz_zip_rw.h"
 
-#include <stdio.h>  /* printf */
+#include <stdio.h> /* printf */
 
 /***************************************************************************/
 
 typedef struct minizip_opt_s {
-    uint8_t     include_path;
-    int16_t     compress_level;
-    uint8_t     compress_method;
-    uint8_t     overwrite;
-    uint8_t     append;
-    int64_t     disk_size;
-    uint8_t     follow_links;
-    uint8_t     store_links;
-    uint8_t     zip_cd;
-    int32_t     encoding;
-    uint8_t     verbose;
-    uint8_t     aes;
-    const char *cert_path;
-    const char *cert_pwd;
+    int64_t disk_size;
+    int32_t encoding;
+    uint8_t include_path;
+    int16_t compress_level;
+    uint8_t compress_method;
+    uint8_t overwrite;
+    uint8_t append;
+    uint8_t follow_links;
+    uint8_t store_links;
+    uint8_t zip_cd;
+    uint8_t verbose;
+    uint8_t aes;
 } minizip_opt;
 
 /***************************************************************************/
@@ -44,7 +42,7 @@ typedef struct minizip_opt_s {
 int32_t minizip_banner(void);
 int32_t minizip_help(void);
 
-int32_t minizip_list(const char *path);
+int32_t minizip_list(const char *path, int32_t encoding);
 
 int32_t minizip_add_entry_cb(void *handle, void *userdata, mz_zip_file *file_info);
 int32_t minizip_add_progress_cb(void *handle, void *userdata, mz_zip_file *file_info, int64_t position);
@@ -54,7 +52,8 @@ int32_t minizip_add(const char *path, const char *password, minizip_opt *options
 int32_t minizip_extract_entry_cb(void *handle, void *userdata, mz_zip_file *file_info, const char *path);
 int32_t minizip_extract_progress_cb(void *handle, void *userdata, mz_zip_file *file_info, int64_t position);
 int32_t minizip_extract_overwrite_cb(void *handle, void *userdata, mz_zip_file *file_info, const char *path);
-int32_t minizip_extract(const char *path, const char *pattern, const char *destination, const char *password, minizip_opt *options);
+int32_t minizip_extract(const char *path, const char *pattern, const char *destination, const char *password,
+                        minizip_opt *options);
 
 int32_t minizip_erase(const char *src_path, const char *target_path, int32_t arg_count, const char **args);
 
@@ -67,47 +66,50 @@ int32_t minizip_banner(void) {
 }
 
 int32_t minizip_help(void) {
-    printf("Usage: minizip [-x][-d dir|-l|-e][-o][-f][-y][-c cp][-a][-0 to -9][-b|-m|-t][-k 512][-p pwd][-s] file.zip [files]\n\n" \
-           "  -x  Extract files\n" \
-           "  -l  List files\n" \
-           "  -d  Destination directory\n" \
-           "  -e  Erase files\n" \
-           "  -o  Overwrite existing files\n" \
-           "  -c  File names use cp437 encoding (or specified codepage)\n" \
-           "  -a  Append to existing zip file\n" \
-           "  -i  Include full path of files\n" \
-           "  -f  Follow symbolic links\n" \
-           "  -y  Store symbolic links\n" \
-           "  -v  Verbose info\n" \
-           "  -0  Store only\n" \
-           "  -1  Compress faster\n" \
-           "  -9  Compress better\n" \
-           "  -k  Disk size in KB\n" \
-           "  -z  Zip central directory\n" \
-           "  -p  Encryption password\n" \
-           "  -s  AES encryption\n" \
-           "  -h  PKCS12 certificate path\n" \
-           "  -w  PKCS12 certificate password\n" \
-           "  -b  BZIP2 compression\n" \
-           "  -m  LZMA compression\n" \
-           "  -n  XZ compression\n" \
-           "  -t  ZSTD compression\n\n");
+    printf(
+        "Usage: minizip [-x][-d dir|-l|-e][-o][-f][-y][-c cp][-a][-0 to -9][-b|-m|-t][-k 512][-p pwd][-s] file.zip "
+        "[files]\n\n"
+        "  -x  Extract files\n"
+        "  -l  List files\n"
+        "  -d  Destination directory\n"
+        "  -e  Erase files\n"
+        "  -o  Overwrite existing files\n"
+        "  -c  File names use cp437 encoding (or specified codepage)\n"
+        "  -a  Append to existing zip file\n"
+        "  -i  Include full path of files\n"
+        "  -f  Follow symbolic links\n"
+        "  -y  Store symbolic links\n"
+        "  -v  Verbose info\n"
+        "  -0  Store only\n"
+        "  -1  Compress faster\n"
+        "  -9  Compress better\n"
+        "  -k  Disk size in KB\n"
+        "  -z  Zip central directory\n"
+        "  -p  Encryption password\n"
+        "  -s  AES encryption\n"
+        "  -b  BZIP2 compression\n"
+        "  -m  LZMA compression\n"
+        "  -n  XZ compression\n"
+        "  -t  ZSTD compression\n\n");
     return MZ_OK;
 }
 
 /***************************************************************************/
 
-int32_t minizip_list(const char *path) {
+int32_t minizip_list(const char *path, int32_t encoding) {
     mz_zip_file *file_info = NULL;
     uint32_t ratio = 0;
     int32_t err = MZ_OK;
     struct tm tmu_date;
     const char *method = NULL;
+    char *utf8_string = NULL;
     char crypt = ' ';
     void *reader = NULL;
 
+    reader = mz_zip_reader_create();
+    if (!reader)
+        return MZ_MEM_ERROR;
 
-    mz_zip_reader_create(&reader);
     err = mz_zip_reader_open_file(reader, path);
     if (err != MZ_OK) {
         printf("Error %" PRId32 " opening archive %s\n", err, path);
@@ -115,6 +117,8 @@ int32_t minizip_list(const char *path) {
         return err;
     }
 
+    mz_zip_reader_set_encoding(reader, encoding);
+
     err = mz_zip_reader_goto_first_entry(reader);
 
     if (err != MZ_OK && err != MZ_END_OF_LIST) {
@@ -127,7 +131,7 @@ int32_t minizip_list(const char *path) {
     printf("      ------     -------- ----- ------   ------- ----     ----  ------     ----\n");
 
     /* Enumerate all entries in the archive */
-    do {
+    while (err == MZ_OK) {
         err = mz_zip_reader_entry_get_info(reader, &file_info);
 
         if (err != MZ_OK) {
@@ -148,15 +152,25 @@ int32_t minizip_list(const char *path) {
         method = mz_zip_get_compression_method_string(file_info->compression_method);
         mz_zip_time_t_to_tm(file_info->modified_date, &tmu_date);
 
+        if ((encoding > 0) && (file_info->flag & MZ_ZIP_FLAG_UTF8) == 0) {
+            utf8_string = mz_os_utf8_string_create(file_info->filename, encoding);
+            if (!utf8_string) {
+                err = MZ_MEM_ERROR;
+                printf("Error %" PRId32 " creating UTF-8 string\n", err);
+                break;
+            }
+        }
+
         /* Print entry information */
-        printf("%12" PRId64 " %12" PRId64 "  %3" PRIu32 "%% %6s%c %8" PRIx32 " %2.2" PRIu32 \
-               "-%2.2" PRIu32 "-%2.2" PRIu32 " %2.2" PRIu32 ":%2.2" PRIu32 " %8.8" PRIx32 "   %s\n",
-               file_info->compressed_size, file_info->uncompressed_size, ratio,
-               method, crypt, file_info->external_fa,
-               (uint32_t)tmu_date.tm_mon + 1, (uint32_t)tmu_date.tm_mday,
-               (uint32_t)tmu_date.tm_year % 100,
-               (uint32_t)tmu_date.tm_hour, (uint32_t)tmu_date.tm_min,
-               file_info->crc, file_info->filename);
+        printf("%12" PRId64 " %12" PRId64 "  %3" PRIu32 "%% %6s%c %8" PRIx32 " %2.2" PRIu32 "-%2.2" PRIu32
+               "-%2.2" PRIu32 " %2.2" PRIu32 ":%2.2" PRIu32 " %8.8" PRIx32 "   %s\n",
+               file_info->compressed_size, file_info->uncompressed_size, ratio, method, crypt, file_info->external_fa,
+               (uint32_t)tmu_date.tm_mon + 1, (uint32_t)tmu_date.tm_mday, (uint32_t)tmu_date.tm_year % 100,
+               (uint32_t)tmu_date.tm_hour, (uint32_t)tmu_date.tm_min, file_info->crc,
+               utf8_string ? utf8_string : file_info->filename);
+
+        if (utf8_string)
+            mz_os_utf8_string_delete(&utf8_string);
 
         err = mz_zip_reader_goto_next_entry(reader);
 
@@ -164,7 +178,7 @@ int32_t minizip_list(const char *path) {
             printf("Error %" PRId32 " going to next entry in archive\n", err);
             break;
         }
-    } while (err == MZ_OK);
+    }
 
     mz_zip_reader_delete(&reader);
 
@@ -200,9 +214,10 @@ int32_t minizip_add_progress_cb(void *handle, void *userdata, mz_zip_file *file_
         progress = ((double)position / file_info->uncompressed_size) * 100;
 
     /* Print the progress of the current compress operation */
-    if (options->verbose)
+    if (options->verbose) {
         printf("%s - %" PRId64 " / %" PRId64 " (%.02f%%)\n", file_info->filename, position,
-            file_info->uncompressed_size, progress);
+               file_info->uncompressed_size, progress);
+    }
     return MZ_OK;
 }
 
@@ -235,18 +250,21 @@ int32_t minizip_add_overwrite_cb(void *handle, void *userdata, const char *path)
     return MZ_OK;
 }
 
-int32_t minizip_add(const char *path, const char *password, minizip_opt *options, int32_t arg_count, const char **args) {
+int32_t minizip_add(const char *path, const char *password, minizip_opt *options, int32_t arg_count,
+                    const char **args) {
     void *writer = NULL;
     int32_t err = MZ_OK;
     int32_t err_close = MZ_OK;
     int32_t i = 0;
     const char *filename_in_zip = NULL;
 
-
     printf("Archive %s\n", path);
 
     /* Create zip writer */
-    mz_zip_writer_create(&writer);
+    writer = mz_zip_writer_create();
+    if (!writer)
+        return MZ_MEM_ERROR;
+
     mz_zip_writer_set_password(writer, password);
     mz_zip_writer_set_aes(writer, options->aes);
     mz_zip_writer_set_compress_method(writer, options->compress_method);
@@ -257,8 +275,6 @@ int32_t minizip_add(const char *path, const char *password, minizip_opt *options
     mz_zip_writer_set_progress_cb(writer, options, minizip_add_progress_cb);
     mz_zip_writer_set_entry_cb(writer, options, minizip_add_entry_cb);
     mz_zip_writer_set_zip_cd(writer, options->zip_cd);
-    if (options->cert_path)
-        mz_zip_writer_set_certificate(writer, options->cert_path, options->cert_pwd);
 
     err = mz_zip_writer_open_file(writer, path, options->disk_size, options->append);
 
@@ -288,12 +304,23 @@ int32_t minizip_add(const char *path, const char *password, minizip_opt *options
 /***************************************************************************/
 
 int32_t minizip_extract_entry_cb(void *handle, void *userdata, mz_zip_file *file_info, const char *path) {
-    MZ_UNUSED(handle);
-    MZ_UNUSED(userdata);
+    minizip_opt *options = (minizip_opt *)userdata;
+    char *utf8_string = NULL;
+
     MZ_UNUSED(path);
 
+    if ((options->encoding > 0) && (file_info->flag & MZ_ZIP_FLAG_UTF8) == 0) {
+        utf8_string = mz_os_utf8_string_create(file_info->filename, options->encoding);
+        if (!utf8_string)
+            return MZ_MEM_ERROR;
+    }
+
     /* Print the current entry extracting */
-    printf("Extracting %s\n", file_info->filename);
+    printf("Extracting %s\n", utf8_string ? utf8_string : file_info->filename);
+
+    if (utf8_string)
+        mz_os_utf8_string_delete(&utf8_string);
+
     return MZ_OK;
 }
 
@@ -302,8 +329,6 @@ int32_t minizip_extract_progress_cb(void *handle, void *userdata, mz_zip_file *f
     double progress = 0;
     uint8_t raw = 0;
 
-    MZ_UNUSED(userdata);
-
     mz_zip_reader_get_raw(handle, &raw);
 
     if (raw && file_info->compressed_size > 0)
@@ -312,9 +337,10 @@ int32_t minizip_extract_progress_cb(void *handle, void *userdata, mz_zip_file *f
         progress = ((double)position / file_info->uncompressed_size) * 100;
 
     /* Print the progress of the current extraction */
-    if (options->verbose)
+    if (options->verbose) {
         printf("%s - %" PRId64 " / %" PRId64 " (%.02f%%)\n", file_info->filename, position,
-            file_info->uncompressed_size, progress);
+               file_info->uncompressed_size, progress);
+    }
 
     return MZ_OK;
 }
@@ -347,7 +373,8 @@ int32_t minizip_extract_overwrite_cb(void *handle, void *userdata, mz_zip_file *
     return MZ_OK;
 }
 
-int32_t minizip_extract(const char *path, const char *pattern, const char *destination, const char *password, minizip_opt *options) {
+int32_t minizip_extract(const char *path, const char *pattern, const char *destination, const char *password,
+                        minizip_opt *options) {
     void *reader = NULL;
     int32_t err = MZ_OK;
     int32_t err_close = MZ_OK;
@@ -355,7 +382,10 @@ int32_t minizip_extract(const char *path, const char *pattern, const char *desti
     printf("Archive %s\n", path);
 
     /* Create zip reader */
-    mz_zip_reader_create(&reader);
+    reader = mz_zip_reader_create();
+    if (!reader)
+        return MZ_MEM_ERROR;
+
     mz_zip_reader_set_pattern(reader, pattern, 1);
     mz_zip_reader_set_password(reader, password);
     mz_zip_reader_set_encoding(reader, options->encoding);
@@ -416,8 +446,14 @@ int32_t minizip_erase(const char *src_path, const char *target_path, int32_t arg
         target_path_ptr = tmp_path;
     }
 
-    mz_zip_reader_create(&reader);
-    mz_zip_writer_create(&writer);
+    reader = mz_zip_reader_create();
+    if (!reader)
+        return MZ_MEM_ERROR;
+    writer = mz_zip_writer_create();
+    if (!writer) {
+        mz_zip_reader_delete(&reader);
+        return MZ_MEM_ERROR;
+    }
 
     /* Open original archive we want to erase an entry in */
     err = mz_zip_reader_open_file(reader, src_path);
@@ -560,56 +596,40 @@ int main(int argc, const char *argv[]) {
             else if ((c == 'v') || (c == 'V'))
                 options.verbose = 1;
             else if ((c >= '0') && (c <= '9')) {
-                options.compress_level = (c - '0');
+                options.compress_level = (int16_t)atoi(&argv[i][1]);
                 if (options.compress_level == 0)
                     options.compress_method = MZ_COMPRESS_METHOD_STORE;
             } else if ((c == 'b') || (c == 'B'))
-#ifdef HAVE_BZIP2
+#  ifdef HAVE_BZIP2
                 options.compress_method = MZ_COMPRESS_METHOD_BZIP2;
-#else
+#  else
                 err = MZ_SUPPORT_ERROR;
-#endif
+#  endif
             else if ((c == 'm') || (c == 'M'))
-#ifdef HAVE_LZMA
+#  ifdef HAVE_LZMA
                 options.compress_method = MZ_COMPRESS_METHOD_LZMA;
-#else
+#  else
                 err = MZ_SUPPORT_ERROR;
-#endif
+#  endif
             else if ((c == 'n') || (c == 'N'))
-#if defined(HAVE_LZMA) || defined(HAVE_LIBCOMP)
+#  if defined(HAVE_LZMA) || defined(HAVE_LIBCOMP)
                 options.compress_method = MZ_COMPRESS_METHOD_XZ;
-#else
+#  else
                 err = MZ_SUPPORT_ERROR;
-#endif
+#  endif
             else if ((c == 't') || (c == 'T'))
-#ifdef HAVE_ZSTD
+#  ifdef HAVE_ZSTD
                 options.compress_method = MZ_COMPRESS_METHOD_ZSTD;
-#else
+#  else
                 err = MZ_SUPPORT_ERROR;
-#endif
+#  endif
             else if ((c == 's') || (c == 'S'))
-#ifdef HAVE_WZAES
+#  ifdef HAVE_WZAES
                 options.aes = 1;
-#else
+#  else
                 err = MZ_SUPPORT_ERROR;
-#endif
-            else if (((c == 'h') || (c == 'H')) && (i + 1 < argc)) {
-#ifdef MZ_ZIP_SIGNING
-                options.cert_path = argv[i + 1];
-                printf("%s ", argv[i + 1]);
-#else
-                err = MZ_SUPPORT_ERROR;
-#endif
-                i += 1;
-            } else if (((c == 'w') || (c == 'W')) && (i + 1 < argc)) {
-#ifdef MZ_ZIP_SIGNING
-                options.cert_pwd = argv[i + 1];
-                printf("%s ", argv[i + 1]);
-#else
-                err = MZ_SUPPORT_ERROR;
-#endif
-                i += 1;
-            } else if (((c == 'c') || (c == 'C')) && (i + 1 < argc)) {
+#  endif
+            else if (((c == 'c') || (c == 'C')) && (i + 1 < argc)) {
                 options.encoding = (int32_t)atoi(argv[i + 1]);
                 i += 1;
             } else if (((c == 'k') || (c == 'K')) && (i + 1 < argc)) {
@@ -621,12 +641,12 @@ int main(int argc, const char *argv[]) {
                 printf("%s ", argv[i + 1]);
                 i += 1;
             } else if (((c == 'p') || (c == 'P')) && (i + 1 < argc)) {
-#ifndef MZ_ZIP_NO_ENCRYPTION
+#  ifndef MZ_ZIP_NO_ENCRYPTION
                 password = argv[i + 1];
                 printf("*** ");
-#else
+#  else
                 err = MZ_SUPPORT_ERROR;
-#endif
+#  endif
                 i += 1;
             }
         } else if (path_arg == 0)
@@ -648,7 +668,7 @@ int main(int argc, const char *argv[]) {
 
     if (do_list) {
         /* List archive contents */
-        err = minizip_list(path);
+        err = minizip_list(path, options.encoding);
     } else if (do_extract) {
         if (argc > path_arg + 1)
             filename_to_extract = argv[path_arg + 1];
